import pandas as pd
import numpy as np
import datetime
import time

from sklearn.linear_model import LinearRegression
from sklearn import preprocessing
from sklearn.model_selection import train_test_split

from matplotlib import pyplot as plt
from sklearn import neural_network
from sklearn.svm import SVR
from sklearn.kernel_ridge import KernelRidge
from datetime import timedelta
import matplotlib.dates as mdates

from sklearn.neighbors import KNeighborsRegressor
# from sklearn.gaussian_process import GaussianProcessRegressor

from sklearn.model_selection import validation_curve
from sklearn import tree

# Setting prediction length 266 = 1 day
num_days = 1
forecast_out = int(num_days * 266)  # predicting * days into future
oneWeek = int(num_days * 266 * 10)
# import scipy.sparse as sparse
df = pd.read_csv('dataset3-8.csv')

# Changing dates to usable format(float)
list_dates = []
for index, row in df.iterrows():
    cleantd = row['timestamp'].split('.')
    dateAndTime = datetime.datetime.strptime(cleantd[0], '%Y-%m-%d %H:%M:%S')
    td = mdates.date2num(dateAndTime)
    list_dates.append(td)
dates = np.array(list_dates)
P = dates[-forecast_out:]

df['dates'] = dates.tolist()
df['Prediction'] = df[['batteryPercent']].shift(-forecast_out)  # label column with data shifted

X = df[['seconds', 'solar', 'batteryPercent']]
y = np.array(df['Prediction'])


X = X[-oneWeek:]
y = y[-oneWeek:]

# Setting plotting and predicting variables #
y_plot_real = np.array(df['batteryPercent'])[-forecast_out:]  # Real points to compare with predicted
y_plot_last = np.array(df['batteryPercent'])[:-forecast_out]  # Last * days points to make plot readable
X_forecast = X[-forecast_out:]  # set X_forecast equal to last * points

# Removing last * points
dates = dates[:-forecast_out]
X = X[:-forecast_out]  # remove last from X
y = y[:-forecast_out]
# X = preprocessing.scale(X)

# Set train and test variables
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2)

# Training and predicting #
clf = LinearRegression()
clf.fit(X_train, y_train)

confidence = clf.score(X_test, y_test)
print("confidence LinearRegression: ", confidence)

forecast_prediction = clf.predict(X_forecast)
# print(forecast_prediction)

# Decision tree #
clf = tree.DecisionTreeRegressor(max_depth=8)
clf = clf.fit(X_train, y_train)

confidence = clf.score(X_test, y_test)
print('DTR:', confidence)
DT_prediction = clf.predict(X_forecast)


# KernelRidge
krP = KernelRidge(alpha=0.04, kernel='rbf', gamma=0.000001)
krP.fit(X_train, y_train)

confidence = krP.score(X_test, y_test)
print('KernelRidge confidence:', confidence)

KernelRidge_prediction = krP.predict(X_forecast)
# print('KernelRidge predictions:', KernelRidge_prediction)

# SVR #
svr = SVR(C=1.0, epsilon=0.1, kernel='rbf', gamma=0.00001)
svr.fit(X_train, y_train)

confidence = svr.score(X_test, y_test)
print('SVR confidence:', confidence)

SVR_predictions = svr.predict(X_forecast)
# print('SVR predictions:', SVR_predictions)

# KNN
neigh = KNeighborsRegressor(n_neighbors=10)
neigh.fit(X_train, y_train)

confidence = neigh.score(X_test, y_test)
print('KNN confidence:', confidence)

KNN_predictions = neigh.predict(X_forecast)

# MLP
MLP = neural_network.MLPRegressor()
MLP.fit(X_train, y_train)

confidence = MLP.score(X_test, y_test)
print('MLP confidence:', confidence)

MLP_predictions = MLP.predict(X_forecast)

# # GPR
# gp = GaussianProcessRegressor(kernel=None,
#                               alpha=1e-30)
# gp.fit(X_train, y_train)
# confidence = gp.score(X_test, y_test)
# print('GPR confidence:', confidence)

# GPR_prediction = gp.predict(X_forecast)


# # Plotting #
fig, ax = plt.subplots(figsize=(20, 10))
ax.plot(dates, y_plot_last, 'bo', markersize=4, label='Training data')

# rotates and right aligns the x labels, and moves the bottom of the
# axes up to make room for them
fig.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)

fig1, ax = plt.subplots(figsize=(20, 10))

ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')

# ax.plot(P, forecast_prediction, 'yo', markersize=4, label='Linear Prediction')
# ax.plot(P, MLP_predictions, 'go', markersize=4, label='MLP Prediction')
ax.plot(P, SVR_predictions, 'ro', markersize=4, label='SVR Prediction')
ax.plot(P, KNN_predictions, 'co', markersize=4, label='KNN Prediction')
ax.plot(P, KernelRidge_prediction, 'mo', markersize=4, label='Kernel Ridge Prediction')
# ax.scatter(P, DT_prediction, color='brown', s=16, label='DTR Prediction')


ax.set_title('Prediction with SVR, Kernel Ridge and KNN')
ax.set_xlabel('Date and Time')
ax.set_ylabel('Battery Percent')
ax.legend()
fig1.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)

fig2, ax = plt.subplots(figsize=(20, 10))

ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')

ax.plot(P, forecast_prediction, 'yo', markersize=4, label='Linear Prediction')
ax.plot(P, MLP_predictions, 'go', markersize=4, label='MLP Prediction')
ax.plot(P, SVR_predictions, 'ro', markersize=4, label='SVR Prediction')
ax.plot(P, KNN_predictions, 'co', markersize=4, label='KNN Prediction')
ax.plot(P, KernelRidge_prediction, 'mo', markersize=4, label='Kernel Ridge Prediction')
ax.scatter(P, DT_prediction, color='brown', s=16, label='DTR Prediction')


ax.set_title('Prediction with SVR, Kernel Ridge, KNN, DTR, MLP and Linear Regression')
ax.set_xlabel('Date and Time')
ax.set_ylabel('Battery Percent')
ax.legend()
fig2.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)


fig3, ax = plt.subplots(figsize=(20, 10))

ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')
ax.plot(P, forecast_prediction, 'yo', markersize=4, label='Linear Prediction')

ax.set_title('Prediction with Linear Regression')
ax.set_xlabel('Date and Time')
ax.set_ylabel('Battery Percent')
ax.legend()
fig3.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)


fig4, ax = plt.subplots(figsize=(20, 10))

ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')
ax.plot(P, SVR_predictions, 'ro', markersize=4, label='SVR Prediction')

ax.set_title('Prediction with SVR')
ax.set_xlabel('Date and Time')
ax.set_ylabel('Battery Percent')
ax.legend()
fig4.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)


fig5, ax = plt.subplots(figsize=(20, 10))

ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')

ax.plot(P, KernelRidge_prediction, 'mo', markersize=4, label='Kernel Ridge Prediction')

ax.set_title('Prediction with Kernel Ridge')
ax.set_xlabel('Date and Time')
ax.set_ylabel('Battery Percent')
ax.legend()
fig5.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)


fig6, ax = plt.subplots(figsize=(20, 10))

ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')

ax.plot(P, MLP_predictions, 'go', markersize=4, label='MLP Prediction')

ax.set_title('Prediction with MLP')
ax.set_xlabel('Date and Time')
ax.set_ylabel('Battery Percent')
ax.legend()
fig6.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)


fig7, ax = plt.subplots(figsize=(20, 10))

ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')

ax.plot(P, KNN_predictions, 'co', markersize=4, label='KNN Prediction')

ax.set_title('Prediction with KNN')
ax.set_xlabel('Date and Time')
ax.set_ylabel('Battery Percent')
ax.legend()
fig7.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)


# fig8, ax = plt.subplots(figsize=(20, 10))

# ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
# ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')

# ax.scatter(P, GPR_prediction, color='brown', s=16, label='GPR Prediction')

# ax.set_title('Prediction with GPR')
# ax.set_xlabel('Date and Time')
# ax.set_ylabel('Battery Percent')
# ax.legend()
# fig8.autofmt_xdate()
# xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
# ax.xaxis.set_major_formatter(xfmt)


fig9, ax = plt.subplots(figsize=(20, 10))
ax.plot(dates[-forecast_out:], y_plot_last[-forecast_out:], 'bo', markersize=4, label='Training data')
ax.plot(P, y_plot_real, 'ko', markersize=4, label='Actual data')
ax.scatter(P, DT_prediction, color='brown', s=16, label='DTR Prediction')

ax.set_title('Prediction with DTR')
ax.set_xlabel('Date and Time')
ax.set_ylabel('Battery Percent')
ax.legend()
fig9.autofmt_xdate()
xfmt = mdates.DateFormatter('%d-%m-%y %H:%M')
ax.xaxis.set_major_formatter(xfmt)


# # # Saving figs
# fig.savefig('Bilder/Plot 3-8/Several Inputs/Training data SevInput.png', dpi=100)
# fig1.savefig('Bilder/Plot 3-8/Several Inputs/Prediction with SVR, Kernel Ridge and KNN SevInput.png', dpi=100)
# fig2.savefig('Bilder/Plot 3-8/Several Inputs/Prediction with ALL SevInput.png', dpi=100)
# fig3.savefig('Bilder/Plot 3-8/Several Inputs/Prediction with Linear Regression SevInput.png', dpi=100)
# fig4.savefig('Bilder/Plot 3-8/Several Inputs/Prediction with SVR SevInput.png', dpi=100)
# fig5.savefig('Bilder/Plot 3-8/Several Inputs/Prediction with Kernel Ridge SevInput.png', dpi=100)
# fig6.savefig('Bilder/Plot 3-8/Several Inputs/Prediction with MLP SevInput.png', dpi=100)
# fig7.savefig('Bilder/Plot 3-8/Several Inputs/Prediction with KNN SevInput.png', dpi=100)
# # fig8.savefig('Bilder/Plot 3-8/Prediction with GPR SevInput.png', dpi=100)
# fig9.savefig('Bilder/Plot 3-8/Several Inputs/Prediction with DTR SevInput.png', dpi=100)

# Test plot score
t0 = time.time()
SVR_time = time.time() - t0
param_range = np.logspace(-12, 1, 5)
train_scores, test_scores = validation_curve(
    SVR(), X, y, param_name="gamma", param_range=param_range,
    cv=10, scoring='r2', n_jobs=1)
train_scores_mean = np.mean(train_scores, axis=1)
train_scores_std = np.std(train_scores, axis=1)
test_scores_mean = np.mean(test_scores, axis=1)
test_scores_std = np.std(test_scores, axis=1)

print("SVR complexity and bandwidth selected and model fitted in %.3f s" % SVR_time)

fig11, ax = plt.subplots(figsize=(20, 10))
ax.set_title("Validation Curve with SVR")
ax.set_xlabel("$\gamma$")
ax.set_ylabel("Score")
ax.set_ylim(0.0, 1.1)
lw = 2
ax.semilogx(param_range, train_scores_mean, label="Training score", color="darkorange", lw=lw)
ax.fill_between(param_range, train_scores_mean - train_scores_std, train_scores_mean + train_scores_std, alpha=0.2, color="darkorange", lw=lw)
ax.semilogx(param_range, test_scores_mean, label="Cross-validation score", color="navy", lw=lw)
ax.fill_between(param_range, test_scores_mean - test_scores_std,
                test_scores_mean + test_scores_std, alpha=0.2,
                color="navy", lw=lw)
ax.legend(loc="best")

fig11.savefig('Bilder/Plot 3-8/Several Inputs/Validation Curve with SVR.png', dpi=100)


plt.show()
